Module: Exwiw

Defined in:
lib/exwiw/adapter/mongodb_adapter.rb,
lib/exwiw.rb,
lib/exwiw/cli.rb,
lib/exwiw/runner.rb,
lib/exwiw/adapter.rb,
lib/exwiw/railtie.rb,
lib/exwiw/version.rb,
lib/exwiw/query_ast.rb,
lib/exwiw/belongs_to.rb,
lib/exwiw/mongo_query.rb,
lib/exwiw/table_column.rb,
lib/exwiw/table_config.rb,
lib/exwiw/mongodb_field.rb,
lib/exwiw/schema_generator.rb,
lib/exwiw/query_ast_builder.rb,
lib/exwiw/adapter/mysql2_adapter.rb,
lib/exwiw/adapter/sqlite3_adapter.rb,
lib/exwiw/mongodb_collection_config.rb,
lib/exwiw/adapter/postgresql_adapter.rb,
lib/exwiw/determine_table_processing_order.rb

Overview

NOTE: This adapter consumes MongodbCollectionConfig (‘fields` instead of the SQL adapters’ ‘columns`). It assumes a “flat” document schema where references between collections are expressed as scalar foreign keys (e.g. `shop_id` on `users`); the forward fan-out strategy here cannot follow references that live inside embedded structures.

Defined Under Namespace

Modules: Adapter, DetermineTableProcessingOrder, MongoQuery, QueryAst Classes: BelongsTo, CLI, ConnectionConfig, DumpTarget, MongodbCollectionConfig, MongodbField, QueryAstBuilder, Railtie, Runner, SchemaGenerator, TableColumn, TableConfig

Constant Summary collapse

VERSION =
"0.1.6"