Class: Exwiw::Adapter::MysqlAdapter
Defined Under Namespace
Classes: StreamingResult
Constant Summary
SqlBulkInsert::STREAM_FLUSH_ROWS
Instance Attribute Summary
Attributes inherited from Base
#connection_config
Instance Method Summary
collapse
#to_bulk_insert, #write_inserts
Methods inherited from Base
#commented_sql, #default_bulk_insert_chunk_size, #describe_query, #dumpable?, #initialize, #output_extension, #query_comment_text, #schema_output_extension, #sql_query_comment, #supports_bulk_delete?, table_config_class, #to_copy_from_stdin, #validate_as_dump_target!, #write_inserts
Instance Method Details
#build_query(table, dump_target, table_by_name) ⇒ Object
56
57
58
|
# File 'lib/exwiw/adapter/mysql_adapter.rb', line 56
def build_query(table, dump_target, table_by_name)
Exwiw::QueryAstBuilder.run(table.name, table_by_name, dump_target, @logger)
end
|
#compile_ast(query_ast, count_only: false) ⇒ Object
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
|
# File 'lib/exwiw/adapter/mysql_adapter.rb', line 229
def compile_ast(query_ast, count_only: false)
raise NotImplementedError unless query_ast.is_a?(Exwiw::QueryAst::Select)
sql = "SELECT "
sql += if count_only
"COUNT(*)"
elsif query_ast.select_all
"*"
else
query_ast.columns.map { |col| compile_column_name(query_ast, col) }.join(', ')
end
sql += " FROM #{query_ast.from_table_name}"
query_ast.join_clauses.each do |join|
sql += " JOIN #{join.join_table_name} ON #{join.base_table_name}.#{join.foreign_key} = #{join.join_table_name}.#{join.primary_key}"
join.where_clauses.each do |where|
compiled_where_condition = compile_where_condition(where, join.join_table_name)
sql += " AND #{compiled_where_condition}"
end
join.base_where_clauses.each do |where|
compiled_where_condition = compile_where_condition(where, join.base_table_name)
sql += " AND #{compiled_where_condition}"
end
end
if query_ast.where_clauses.any?
sql += " WHERE "
sql += query_ast.where_clauses.map { |where| compile_where_condition(where, query_ast.from_table_name) }.join(' AND ')
end
sql
end
|
#dump_schema(ordered_tables, output_path) ⇒ Object
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
|
# File 'lib/exwiw/adapter/mysql_adapter.rb', line 82
def dump_schema(ordered_tables, output_path)
table_names = ordered_tables.map(&:name)
if table_names.empty?
File.write(output_path, "-- Auto-generated by exwiw. No tables in scope.\n")
return
end
mysqldump_bin = ENV['EXWIW_MYSQLDUMP']
mysqldump_bin = 'mysqldump' if mysqldump_bin.nil? || mysqldump_bin.empty?
gtid_flags = mariadb_mysqldump?(mysqldump_bin) ? [] : ['--set-gtid-purged=OFF']
cmd = [
mysqldump_bin,
"--host=#{@connection_config.host}",
"--port=#{@connection_config.port}",
"--user=#{@connection_config.user}",
'--no-data',
'--skip-add-drop-table',
'--skip-comments',
'--skip-set-charset',
'--no-tablespaces', *gtid_flags,
'--compact',
@connection_config.database_name,
*table_names,
]
env = { 'MYSQL_PWD' => @connection_config.password.to_s }
@logger.debug(" Running #{mysqldump_bin} for #{table_names.size} table(s)...")
stdout, stderr, status =
begin
Open3.capture3(env, *cmd)
rescue Errno::ENOENT
raise "Failed to run `#{mysqldump_bin}`. Ensure the mysql client is installed and on PATH, " \
"or set EXWIW_MYSQLDUMP to a mysqldump binary."
end
unless status.success?
if stderr.include?('command not found') || stderr.empty?
raise "Failed to run `#{mysqldump_bin}`. Ensure the mysql client is installed and on PATH, " \
"or set EXWIW_MYSQLDUMP to a mysqldump binary. stderr: #{stderr}"
end
raise "mysqldump failed (exit #{status.exitstatus}): #{stderr}"
end
idempotent = DdlPostprocessor.add_if_not_exists_to_create_table(stdout)
File.open(output_path, 'w') do |file|
file.puts("-- Auto-generated by exwiw via mysqldump. Idempotent CREATE TABLE statements for mysql.")
file.puts("SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0;")
file.puts
file.puts(idempotent)
file.puts("SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS;")
end
@logger.info(" Wrote schema for #{table_names.size} table(s) to #{output_path}.")
end
|
#execute(query_ast) ⇒ Object
60
61
62
63
64
65
66
67
68
69
|
# File 'lib/exwiw/adapter/mysql_adapter.rb', line 60
def execute(query_ast)
data_sql = (query_ast)
count_sql = "#{(query_ast)} #{compile_ast(query_ast, count_only: true)}"
@logger.debug(" Executing SQL (streaming): \n#{data_sql}")
StreamingResult.new(client: connection, data_sql: data_sql, count_sql: count_sql)
end
|
#explain(query_ast) ⇒ Object
71
72
73
74
75
76
77
78
79
80
|
# File 'lib/exwiw/adapter/mysql_adapter.rb', line 71
def explain(query_ast)
sql = (query_ast)
@logger.debug(" Executing EXPLAIN: \n#{sql}")
result = connection.query("EXPLAIN #{sql}")
result.rows.each_with_index.flat_map do |row, i|
["*************************** #{i + 1}. row ***************************"] +
result.fields.zip(row).map { |k, v| "#{k}: #{v}" }
end.join("\n")
end
|
#post_insert_sql(_table) ⇒ Object
150
151
152
|
# File 'lib/exwiw/adapter/mysql_adapter.rb', line 150
def post_insert_sql(_table)
"SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS;"
end
|
#pre_insert_sql(_table) ⇒ Object
146
147
148
|
# File 'lib/exwiw/adapter/mysql_adapter.rb', line 146
def pre_insert_sql(_table)
"SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0;"
end
|
#to_bulk_delete(select_query_ast, table) ⇒ Object
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
|
# File 'lib/exwiw/adapter/mysql_adapter.rb', line 167
def to_bulk_delete(select_query_ast, table)
raise NotImplementedError unless select_query_ast.is_a?(Exwiw::QueryAst::Select)
sql = "DELETE FROM #{select_query_ast.from_table_name}"
if select_query_ast.join_clauses.empty?
compiled_where_conditions = select_query_ast.
where_clauses.
select { |where| where.is_a?(Exwiw::QueryAst::WhereClause) }.
map do |where|
compile_where_condition(where, select_query_ast.from_table_name)
end
if compiled_where_conditions.size > 0
sql += "\nWHERE "
sql += compiled_where_conditions.join(' AND ')
end
sql += ";"
return sql
end
subquery_ast = Exwiw::QueryAst::Select.new
first_join = select_query_ast.join_clauses.first.clone
subquery_ast.from(first_join.join_table_name)
primay_key_col = table.columns.find { |col| col.name == table.primary_key }
subquery_ast.select([primay_key_col])
select_query_ast.join_clauses[1..].each do |join|
subquery_ast.join(join)
end
first_join.where_clauses.each do |where|
subquery_ast.where(where) if where.is_a?(Exwiw::QueryAst::WhereClause)
end
foreign_key = first_join.foreign_key
subquery_sql = compile_ast(subquery_ast)
sql += "\nWHERE #{select_query_ast.from_table_name}.#{foreign_key} IN (#{subquery_sql})"
first_join.base_where_clauses.each do |where|
next unless where.is_a?(Exwiw::QueryAst::WhereClause)
sql += " AND #{compile_where_condition(where, select_query_ast.from_table_name)}"
end
sql += ";"
sql
end
|