Class: Exwiw::TableConfig
- Inherits:
-
Object
- Object
- Exwiw::TableConfig
- Includes:
- Serdes
- Defined in:
- lib/exwiw/table_config.rb
Constant Summary collapse
- RAILS_MANAGED_SCHEMA_MIGRATIONS =
"rails_managed_schema_migrations"- RAILS_MANAGED_INTERNAL_METADATA =
"rails_managed_internal_metadata"- RAILS_MANAGED_TYPES =
[ RAILS_MANAGED_SCHEMA_MIGRATIONS, RAILS_MANAGED_INTERNAL_METADATA, ].freeze
- UNSUPPORTED_COMPOSITE_PRIMARY_KEY =
type marking a table with a composite primary key, which exwiw does not support yet. schema:generate attaches it together with skip:true. Unlike rails-managed tables, columns/belongs_tos are retained so it can serve as a signpost for adding support later.
"unsupported_composite_primary_key"
Class Method Summary collapse
Instance Method Summary collapse
- #belongs_to(table_name) ⇒ Object
- #build_extract_query(extract_target_table, extract_target_ids, tables_by_name) ⇒ Object
- #column_names ⇒ Object
- #merge(passed_table) ⇒ Object
- #rails_managed? ⇒ Boolean
- #to_hash ⇒ Object
Class Method Details
.from(hash) ⇒ Object
30 31 32 33 34 |
# File 'lib/exwiw/table_config.rb', line 30 def self.from(hash) config = super config.send(:validate_after_load!) config end |
.from_symbol_keys(hash) ⇒ Object
36 37 38 |
# File 'lib/exwiw/table_config.rb', line 36 def self.from_symbol_keys(hash) from(JSON.parse(hash.to_json)) end |
Instance Method Details
#belongs_to(table_name) ⇒ Object
57 58 59 |
# File 'lib/exwiw/table_config.rb', line 57 def belongs_to(table_name) belongs_tos.find { |relation| relation.table_name == table_name } end |
#build_extract_query(extract_target_table, extract_target_ids, tables_by_name) ⇒ Object
61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 |
# File 'lib/exwiw/table_config.rb', line 61 def build_extract_query(extract_target_table, extract_target_ids, tables_by_name) # target is itself if name == extract_target_table return [{ from: name, where: [{ primary_key => extract_target_ids }], join: [], select: column_names, }] end # it is not related to target table if belongs_to.empty? return [{ from: name, where: [], join: [], select: column_names, }] end belongs_to_extract_target_table = belongs_tos.find { |relation| relation.table_name == extract_target_table } if belongs_to_extract_target_table key = belongs_to_extract_target_table.foreign_key return [{ from: name, where: [{ key => extract_target_ids }], join: [], select: column_names }] end ret = compute_dependency_to_table(extract_target_table, tables_by_name) if ret.empty? [{ from: name, where: [], join: [], select: column_names, }] else last = ret.last last[:where] = [{ last[:foreign_key] => extract_target_ids }] ret end end |
#column_names ⇒ Object
53 54 55 |
# File 'lib/exwiw/table_config.rb', line 53 def column_names columns.map(&:name) end |
#merge(passed_table) ⇒ Object
104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 |
# File 'lib/exwiw/table_config.rb', line 104 def merge(passed_table) return passed_table if passed_table.to_hash == self.to_hash TableConfig.new.tap do |merged_table| merged_table.name = name merged_table.primary_key = passed_table.primary_key merged_table.type = passed_table.type merged_table.comment = comment merged_table.filter = filter merged_table.belongs_tos = passed_table.belongs_tos merged_table.bulk_insert_chunk_size = passed_table.bulk_insert_chunk_size merged_table.skip = skip receiver_column_by_name = columns.each_with_object({}) { |column, hash| hash[column.name] = column } merged_table.columns = passed_table.columns.map do |passed_column| if receiver_column_by_name.key?(passed_column.name) receiver_column = receiver_column_by_name[passed_column.name] receiver_column else passed_column end end end end |
#rails_managed? ⇒ Boolean
40 41 42 |
# File 'lib/exwiw/table_config.rb', line 40 def rails_managed? RAILS_MANAGED_TYPES.include?(type) end |
#to_hash ⇒ Object
44 45 46 47 48 49 50 51 |
# File 'lib/exwiw/table_config.rb', line 44 def to_hash hash = super if rails_managed? hash.delete("belongs_tos") hash.delete("columns") end hash end |